import type { RawThemeSetting, ThemeRegistrationAny } from 'shiki'
import { StyleSheet, type TextStyle } from 'react-native'

export interface HighlighterStyleSheet {
  container: TextStyle
  token: TextStyle
  selection: TextStyle
  styles: Record<string, TextStyle>
  [key: string]: TextStyle | Record<string, TextStyle>
}

interface TokenStyle {
  color?: string
  fontStyle?: 'normal' | 'italic'
  fontWeight?: 'normal' | 'bold'
}

function processSettings(settings: RawThemeSetting[]): Record<string, TokenStyle> {
  const tokenStyles: Record<string, TokenStyle> = {}

  settings.forEach((setting) => {
    if (!setting.scope || !setting.settings)
      return

    const style: TokenStyle = {}
    const { foreground, fontStyle } = setting.settings

    if (foreground) {
      style.color = foreground
    }

    if (fontStyle) {
      if (fontStyle.includes('italic')) {
        style.fontStyle = 'italic'
      }
      if (fontStyle.includes('bold')) {
        style.fontWeight = 'bold'
      }
    }

    // Handle single scope
    if (typeof setting.scope === 'string') {
      tokenStyles[setting.scope] = style
    }
    // Handle multiple scopes
    else if (Array.isArray(setting.scope)) {
      setting.scope.forEach((scope) => {
        tokenStyles[scope] = style
      })
    }
  })

  return tokenStyles
}

export function convertShikiTheme(theme: ThemeRegistrationAny): ReturnType<typeof StyleSheet.create> {
  // Get all styles from both settings and tokenColors
  const settings = [...(theme.settings || []), ...(theme.tokenColors || [])]

  // Process basic colors from the theme
  const backgroundColor = theme.colors?.['editor.background'] || theme.bg || '#1e1e1e'
  const defaultColor = theme.colors?.['editor.foreground'] || theme.fg || '#d4d4d4'
  const selectionColor = theme.colors?.['editor.selectionBackground'] || 'rgba(255, 255, 255, 0.1)'

  // Process token styles
  const tokenStyles = processSettings(settings)

  // Convert token styles to RN styles
  const tokenStylesMap = Object.entries(tokenStyles).reduce<Record<string, TextStyle>>((acc, [scope, style]) => {
    acc[`token-${scope.replace(/\./g, '-')}`] = {
      color: style.color,
      ...(style.fontStyle && { fontStyle: style.fontStyle }),
      ...(style.fontWeight && { fontWeight: style.fontWeight }),
    }
    return acc
  }, {})

  // Create the complete styles object
  const completeStyles: HighlighterStyleSheet = {
    container: {
      backgroundColor,
      flex: 1,
      padding: 16,
    },
    token: {
      color: defaultColor,
      fontSize: 14,
      lineHeight: 21,
    },
    selection: {
      backgroundColor: selectionColor,
    },
    styles: tokenStylesMap,
  }

  return StyleSheet.create(completeStyles)
}

export function getTokenStyle(theme: HighlighterStyleSheet, tokenTypes: string[], defaultColor: string): TextStyle {
  const matchingStyles = tokenTypes.map(type => theme.styles[`token-${type.replace(/\./g, '-')}`]).filter(Boolean)

  return {
    ...theme.token,
    color: matchingStyles[0]?.color || defaultColor,
    ...(matchingStyles.some(s => s?.fontStyle === 'italic') && { fontStyle: 'italic' }),
    ...(matchingStyles.some(s => s?.fontWeight === 'bold') && { fontWeight: 'bold' }),
  }
}
